/* The "unknown" protocol.  This is what is used for protocols we
 * don't understand.  It's returned by find_proto().
 */

/* FIXME: plug in unknowns if a proto field is NULL upon registration. */

static int unknown_pkt_to_tuple(const void *datah, size_t datalen, 
				struct ip_nat_tuple *tuple)
{
	return 1;
}

static void unknown_tuple_to_pkt(struct iphdr *hdr, 
				 size_t len,
				 const struct ip_nat_tuple_manip *tuple,
				 enum ip_nat_direction direction)
{
	return;
}

static int unknown_in_range(const struct ip_nat_tuple_proto *tuplepr,
			    enum ip_nat_manip_type manip_type,
			    const union ip_nat_proto_manip *min,
			    const union ip_nat_proto_manip *max)
{
	return 1;
}

static int unknown_unique_tuple(struct ip_nat_tuple_proto *tuplepr,
				const struct ip_nat_range *range,
				enum ip_nat_manip_type manip_type)
{
	/* Sorry: we can't help you; if it's not unique, we can't frob
	   anything. */
	return 0;
}

static void unknown_invert_tuple(struct ip_nat_tuple *tuple)
{
}

static unsigned int 
unknown_print_tuple(char *buffer, const struct ip_nat_tuple *tuple)
{
	return 0;
}

static unsigned int 
unknown_print_manip(char *buffer, const union ip_nat_proto_manip *prmanip)
{
	return 0;
}

static unsigned int 
unknown_print_private(char *buffer, const struct ip_nat_binding *binding)
{
	return 0;
}

static int 
unknown_check(const struct ip_nat_rule *rule, unsigned int typenum)
{
	/* Don't let them explicitly create rule of unknown type: use
	 * the generic to ensure they realize it's not supported. */
	DEBUGP("Attempt to create rule with unknown type.\n");
	return 0;
}

static unsigned int 
unknown_rule(const struct iphdr *iph,
	     const struct ip_nat_pktinfo *pkt,
	     const struct ip_nat_rule_element *cause,
	     struct ip_nat_multi_range *mr,
	     unsigned int typenumber,
	     struct ip_nat_binding **binding)
{
	/* We know nothing about this protocol, so keep connections
	   stored for 60 minutes since last packet. */
	*binding = ip_nat_alloc_binding(pkt, cause, mr, 
					&ip_nat_refresh, &ip_nat_refresh, 
					HZ*60*60, 
					&ip_nat_delete_binding_timeout);
	return NF_ACCEPT;
}

static void unknown_destroy(const struct ip_nat_binding *binding)
{
	return;
}

static struct ip_nat_proto unknown_proto = {
	NULL, 0, "unknown",
	unknown_pkt_to_tuple, 
	unknown_tuple_to_pkt, 
	unknown_in_range, 
	unknown_unique_tuple,
	unknown_invert_tuple,
	unknown_print_tuple,
	unknown_print_manip,
	{ NULL, "unknown", 0, NULL, 0, 0, 
	  unknown_print_private,
	  unknown_check, 
	  unknown_rule, 
	  unknown_destroy 
	}
};

