/* Shared library add-on to iptables: MAC matching. */
#include <stdio.h>
#include <getopt.h>
#include <ctype.h>
#include <net/ethernet.h>
#include "libipfwc/libipfwc.h"
#include "iptables.h"

struct mac_hdr {
    int verdict;
    unsigned char addr[ETH_ALEN];
};

static struct option mac_opts[] = {
	{ "mac", 1, 0, '!' },
	{0}
};

void help(void)
{
	fprintf(stderr, "MAC help:\n"
		"  --mac <address> <verdict>\n");
}

struct option *get_options(void)
{
	return mac_opts;
}

static unsigned int hex2nybble(const char *eth, unsigned int i)
{
	if (eth[i] >= '0' && eth[i] <= '9')
		return eth[i] - '0';
	if (toupper(eth[i]) >= 'A' && toupper(eth[i]) <= 'F')
		return toupper(eth[i]) - 'A' + 10;
	exit_error(PARAMETER_PROBLEM, "Bad ethernet address `%s'", eth);
}

/* Expect "00:60:97:92:5A:39" */
static void parse_eth(const char *eth, struct mac_hdr *hdr)
{
	unsigned int i;

	for (i = 0; i < 17; i++) {
		switch (i % 3) {
		case 0:
			hdr->addr[i/3] = hex2nybble(eth, i) << 4;
			break;
		case 1:
			hdr->addr[i/3] |= hex2nybble(eth, i);
			break;
		case 3:
			if (eth[i] != ':')
				exit_error(PARAMETER_PROBLEM,
					   "Bad ethernet address `%s'", eth);
		}
	}
	if (eth[i])
		exit_error(PARAMETER_PROBLEM, "Bad ethernet address `%s'",eth);
}

void parse_target(const char *target, struct mac_hdr *hdr)
{
	if (!target || target[0] == '-')
		exit_error(PARAMETER_PROBLEM, "Expect two args to --mac");

	/* Maybe it's a standard action name... */
	if (strcmp(target, IPFWC_LABEL_ACCEPT) == 0)
		hdr->verdict = -NF_ACCEPT - 1;
	else if (strcmp(target, IPFWC_LABEL_DROP) == 0)
		hdr->verdict = -NF_DROP - 1;
	else if (strcmp(target, IPFWC_LABEL_USER) == 0)
		hdr->verdict = -NF_QUEUE - 1;
	else if (strcmp(target, IPFWC_LABEL_RETURN) == 0)
		hdr->verdict = -IPT_RETURN - 1;
	else
		exit_error(PARAMETER_PROBLEM, "Bad target for --mac: `%s'",
			   target);
}

int parse(int invert, int option, char **argv, struct ipt_entry *entry)
{
	struct mac_hdr *hdr = (struct mac_hdr *)&entry->action;

	switch (option) {
	case '!':
		if (invert)
			exit_error(PARAMETER_PROBLEM, "unexpected ! flag");

		parse_eth(optarg, hdr);
		optind++;
		parse_target(argv[optind-1], hdr);
		break;

	default:
		return 0;
	}

	return 1;
}
