/* Shared library add-on to iptables for standard target support. */
#include <stdio.h>
#include <netdb.h>
#include <string.h>
#include <stdlib.h>
#include <limits.h>
#include "iptables.h"
#include "ipt_tcp.h"

/* Function which prints out usage message. */
static void 
help(void)
{
	printf(
"Standard options:\n"
"(If target is DENY, ACCEPT, RETURN or nothing)\n"
" --mark	-m	mark	number to mark on matching packet\n\n");
}

static struct option opts[] = {
	{ "mark", 1, 0, '1' },
	{0}
};

/* Initialize target; target filled in by libipfwc */
static void
init(union ipt_targinfo *targinfo)
{
	targinfo->standard = ((struct ipt_standard){ 0, 0, { 0, 0 }});
}

static void
parse_mark(const char *markstring, u_int32_t *mark)
{
	char *end;
	unsigned long l;

	l = strtoul(markstring, &end, 0);

	if (l == 0 || end[0] != '\0' || end == markstring || l > UINT_MAX)
		exit_error(PARAMETER_PROBLEM, "Bad value `%s' for --mark.\n",
			   markstring);
}

/* Function which parses command options; returns true if it
   ate an option */
static int 
parse(int c, char **argv, int invert, unsigned int *flags,
      struct ipt_entry *entry)
{
	switch (c) {
	case '1':
		if (check_inverse(optarg, &invert))
			exit_error(PARAMETER_PROBLEM, 
				   "! not allowed with --mark");

		parse_mark(optarg, &entry->targinfo.standard.mark);
		break;

	default:
		return 0;
	}

	return 1;
}

/* Final check; don't care. */
static void final_check(unsigned int flags)
{
}

/* Prints out the union ipt_targinfo. */
static void print(const union ipt_targinfo *info, int numeric)
{
	if (info->standard.mark)
		printf("mark=%u ", info->standard.mark);
}

struct iptables_target standard
= { NULL, 
    "standard", 
    &help, 
    &init,
    &parse,
    &final_check,
    &print,
    opts
};

void _init(void)
{
	register_target(&standard);
}
